// A class for euclidean codes

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// Created: 01/17/1997
// $Id: pskcode.cc,v 1.1 2003/10/14 08:49:02 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: pskcode.cc,v $
// Revision 1.1  2003/10/14 08:49:02  frank
// moved from . to src/
//
// Revision 1.4  2003/10/14 08:41:58  frank
// ANSI C++ / gcc 3.x compatibility changes
//
// Revision 1.3  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.2  1998/05/28 12:56:37  paehlke
// Final version for masters thesis
//
// Revision 1.1  1997/06/20 12:48:55  frank
// Initial revision
//

#ifndef _PSKCODE_H
#include "pskcode.h"
#endif

#ifndef _CODE_H
#include "code.h"
#endif

using std::endl;

PSKCode::PSKCode (istream & is)
{
  int i;
  is >> m >> n >> quant;
  assert (m > 0 && n > 0 && quant > 0);
  cc = new int [mn = m*n];
  cv = new pInt [m];
  for (i=0; i<m; i++)
    cv[i] = &cc[i*n];
  for (i=0; i<mn; i++) {
    is >> cc [i];
    assert (cc[i] >= 0 && cc[i] < quant);
  }
}

PSKCode::PSKCode (const Code & code, int _quant)
{
  int n1 = code.get_n();
  assert (_quant > 0 && (n1&1) == 0); // n1 must be even
  int i,j;
  m = code.get_m();
  n = n1/2;
  cc = new int [mn = m*n];
  cv = new pInt [m];
  for (i=0; i<m; i++)
    cv[i] = &cc[i*n];
  quant = _quant;
  for (i=0; i<m; i++) {
    const Vector & cvi = code[i];
    int * pvi = cv[i];
    for (j=0; j<n; j++) {
      double angle = atan2 (cvi[2*j+1], cvi[2*j]);
      if (angle < 0)
	angle += 2.0*M_PI;
      pvi[j] = (int) floor (quant * angle / (2.0*M_PI));
      if (pvi[j] > quant)
	pvi[j] = quant;
    }
  }
}

PSKCode & PSKCode::Quantize (int qnew)
{
  assert (qnew > 0);
  if (qnew != quant) {
    int i;
    for (i=0; i<mn; i++)
      cc[i] = (cc[i] * qnew) / quant;
    quant = qnew;
  }
  return *this;
}

istream & operator >> (istream & is, PSKCode & c)
{
  int m,n,q,i;
  is >> m >> n >> q;
  assert (m > 0 && n > 0 && q > 0);
  if (m != c.m || n != c.n) {
    if (c.cc != NULL) {
      delete [] c.cc;
      delete [] c.cv;
    }
    c.m = m;
    c.n = n;
    c.cc = new int [c.mn = m*n];
    c.cv = new PSKCode::pInt [m];
    for (i=0; i<m; i++)
      c.cv[i] = &c.cc[i*n];
  }
  c.quant = q;
  for (i=0; i<c.mn; i++) {
    is >> c.cc[i];
    assert (c.cc[i] >= 0 && c.cc[i] < q);
  }
  return is;
}

ostream & operator << (ostream & os, const PSKCode & c)
{
  int i,j;
  os << c.m << ' ' << c.n << ' ' << c.quant << endl;
  for (i=0; i<c.m; i++) {
    const int * cvi = c.cv[i];
    for (j=0; j<c.n; j++)
      os << cvi[j] << ' ';
    os << endl;
  }
  return os;
}
