#ifndef PSKCODE_H
#define PSKCODE_H

// A class for MPSK codes

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// Created: 01/18/1997
// $Id: pskcode.h,v 1.1 2003/10/14 08:49:02 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: pskcode.h,v $
// Revision 1.1  2003/10/14 08:49:02  frank
// moved from . to src/
//
// Revision 1.5  2003/10/14 08:41:58  frank
// ANSI C++ / gcc 3.x compatibility changes
//
// Revision 1.4  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.3  1998/05/28 12:57:04  paehlke
// Final version for masters thesis
//
// Revision 1.2  1997/06/20 14:13:42  frank
// fixed some bugs
//
// Revision 1.1  1997/06/20 12:48:55  frank
// Initial revision
//

#ifndef _ASSERT_H
#include <assert.h>
#endif

#ifndef _MATH_H
#include <math.h>
#endif

#ifndef _CPP_IOSTREAM
#include <iostream>
#endif

#ifndef _VECTOR_H
#include "vector.h"
#endif

#ifndef _CODE_H
class Code;
#endif

using std::istream;
using std::ostream;

// --- BEGIN OF PUBLIC INTERFACE ---

class PSKCode {
public:

  friend class Code;

  PSKCode ();
  PSKCode (int _m, int _n, int _quant);
  PSKCode (const PSKCode &);
  PSKCode (istream &);
  PSKCode (const Code & c, int _quant);

  int get_m () const;
  int get_n () const;
  int get_quant () const;
  int * operator [] (int);
  const int * operator [] (int) const;

  PSKCode & Quantize (int);

  friend istream & operator >> (istream &, PSKCode &);
  friend ostream & operator << (ostream &, const PSKCode &);

// --- END OF PUBLIC INTERFACE ---

  typedef int * pInt;

protected:

  pInt * cv;
  int * cc;
  int m,n,mn,quant;
};

inline PSKCode::PSKCode ()
{
  cv = NULL;
  cc = NULL;
  m = n = mn = quant = 0;
}

inline PSKCode::PSKCode (int _m, int _n, int _quant)
{
  int i;
  assert (_m > 0 && _n > 0 && _quant > 0);
  m = _m;
  n = _n;
  mn = m * n;
  quant = _quant;
  cc = new int [mn];
  cv = new pInt [m];
  for (i=0; i<m; i++)
    cv[i] = &cc[i*n];
  for (i=0; i<mn; i++)
    cc[i] = 0;
}

inline PSKCode::PSKCode (const PSKCode & code)
{
  int i;
  m = code.m;
  n = code.n;
  mn = m * n;
  quant = code.quant;
  cc = new int [mn];
  cv = new pInt [m];
  for (i=0; i<m; i++)
    cv[i] = &cc[i*n];
  for (i=0; i<mn; i++)
    cc[i] = code.cc[i];
}

inline int PSKCode::get_m () const
{
  return m;
}

inline int PSKCode::get_n () const
{
  return n;
}

inline int PSKCode::get_quant () const
{
  return quant;
}

inline int * PSKCode::operator [] (int i)
{
  return cv[i];
}

inline const int * PSKCode::operator [] (int i) const
{
  return cv[i];
}

#endif
