#ifndef _QUICKSORT_H
#define _QUICKSORT_H

// QuickSort routines in C++ (quick and dirty)
// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// $Id: quicksort.h,v 1.1 2003/10/14 08:49:02 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: quicksort.h,v $
// Revision 1.1  2003/10/14 08:49:02  frank
// moved from . to src/
//
// Revision 1.2  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.1  1998/05/28 12:57:04  paehlke
// Final version for masters thesis
//

// QuickSort of array arr[u..o] for ascending key[arr[i]]
// class KEY must provide operators >= and <=

template<class KEY> void QuickSortUp
(int * arr, const KEY * key, int u, int o)
{
  int i,j;
  if (u >= o)
    return;
  int m = (u + o) >> 1;
  int h = arr[u];
  arr[u] = arr[m];
  arr[m] = h;
  double thresh = key[arr[u]];
  for (i = u+1, j = o; i <= j; i++, j--) {
    // bi[u..i-1] <= thresh && bi[j+1..o] >= thresh
    while (i <= j && key[arr[i]] <= thresh)
      i++;
    if (i > j)
      break;
    while (j >= i && key[arr[j]] >= thresh)
      j--;
    if (j < i)
      break;
    // bi[i] > thresh && bi[j] < thresh && u < i <= j <= o
    h = arr[i];
    arr[i] = arr[j];
    arr[j] = h;
    // bi[u..i] <= thresh && bi[j..o] >= thresh
  }
  // bi[u..i-1] <= thresh && bi[j+1..o] >= thresh
  if (i > o) {
    h = arr[u];
    arr[u] = arr[o];
    arr[o] = h;
    QuickSortUp (arr, key, u, o-1);
  }
  else {
    QuickSortUp (arr, key, u, j);
    QuickSortUp (arr, key, i, o);
  }
}

// QuickSort of array arr[u..o] for descending key[arr[i]]
// class KEY must provide operators >= and <=

template<class KEY> void QuickSortDown
(int * arr, const KEY * key, int u, int o)
{
  int i,j;
  if (u >= o)
    return;
  int m = (u+o) >> 1;
  int h = arr[u];
  arr[u] = arr[m];
  arr[m] = h;
  double thresh = key[arr[u]];
  for (i=u+1, j=o; i<=j; i++, j--) {
    // bi[u..i-1] >= thresh && bi[j+1..o] <= thresh
    while (i <= j && key[arr[i]] >= thresh)
      i++;
    if (i > j)
      break;
    while (j >= i && key[arr[j]] <= thresh)
      j--;
    if (j < i)
      break;
    // bi[i] < thresh && bi[j] > thresh && u < i <= j <= o
    h = arr[i];
    arr[i] = arr[j];
    arr[j] = h;
    // bi[u..i] >= thresh && bi[j..o] <= thresh
  }
  // bi[u..i-1] >= thresh && bi[j+1..o] <= thresh
  if (i > o) {
    h = arr[u];
    arr[u] = arr[o];
    arr[o] = h;
    QuickSortDown (arr, key, u, o-1);
  }
  else {
    QuickSortDown (arr, key, u, j);
    QuickSortDown (arr, key, i, o);
  }
}

#endif
