// A program for generating a table of generalized BCH codes over GF(q)

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// $Id: bchcodes.cc,v 1.1 2003/10/14 08:59:11 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: bchcodes.cc,v $
// Revision 1.1  2003/10/14 08:59:11  frank
// moved from . to src/
//
// Revision 1.1  2003/10/14 08:38:59  frank
// renamed bchcodes.c to bchcodes.cc
//
// Revision 1.3  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.2  1998/05/28 12:56:40  paehlke
// Final version for masters thesis
//
// Revision 1.1  1997/06/20 18:48:19  frank
// Initial revision
//

#include <cmath>
#include <iostream>
#include <vector>

using namespace std;

double peb (int n, int t, double p)
{
  int i;
  double logp = log(p), q = 1.0-p, logq = log(q), logb, res;
  logb = 0.0; // logb = log(binomial(n-1,i))
  res = 1.0 - exp (logb + (n-1)*logq);
  for (i=1; i<t; i++) {
	logb = logb + log((double)(n-i)) - log((double)i);
    res -= exp (logb + i*logp + (n-1-i)*logq);
    if (res < 1e-6) {
      res = 0.0;
      break;
    }
  }
  res *= p/2.0;
  return res;
}

double pes (int n, int k)
{
  double lx1,lx2,lxm,pex1,pex2,pexm;
  lx1=log(1e-5); pex1=peb (n,k,1e-5);
  lx2=0.0; pex2=peb (n,k,1.0);
  while (lx2-lx1 >= 1e-5) {
    lxm = (lx1+lx2)/2.0;
    pexm = peb (n,k,exp(lxm));
    if (pexm < 1e-5) {
      lx1 = lxm; pex1 = pexm;
    }
    else {
      lx2 = lxm; pex2 = pexm;
    }
  }
  return exp((lx1+lx2)/2.0);
}

int main (int argc, char ** argv)
{
  int q,emax,i,j,ii,e,qe,n,k;
  vector<unsigned char> arr;
  if (argc < 3)
	  return 1;
  q = atoi (argv[1]);
  emax = atoi (argv[2]);
  cout << "n\tk\td\tt\tpes\n\n";
  for (e=1, qe=q; e<=emax; e++) {
    n = qe-1;
    arr.assign (n, 1);
    i=0; k=n;
    while (k > 0) {
      ii = i;
      for (j=0; j<e; j++) {
	if (arr[ii]) {
	  arr[ii]=0;
	  k--;
	}
	ii = (ii*q) % n;
      }
      if (k == 0)
	break;
      while (arr[i]==0)
		  i++;
	  cout << n << '\t' << k << '\t' << (i+1) << '\t' << (i>>1) << '\t'
		   << pes (n, i>>1) << endl;
    }
    qe *= q;
  }
  return 0;
}
