// Several utilities for managing code files

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// Created: 01/15/1997
// $Id: code-util.cc,v 1.1 2003/10/14 08:59:11 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: code-util.cc,v $
// Revision 1.1  2003/10/14 08:59:11  frank
// moved from . to src/
//
// Revision 1.4  2003/10/14 08:41:58  frank
// ANSI C++ / gcc 3.x compatibility changes
//
// Revision 1.3  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.2  1998/05/28 12:56:07  paehlke
// Final version for masters thesis
//
// Revision 1.1  1997/06/20 18:03:58  paehlke
// Initial revision
//

#include <math.h>
#include <ctype.h>
#include <fstream>
#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include "code.h"
#include "pskcode.h"

using namespace std;

char psk2code_usage[] =
"Usage: psk2code [OPTION...] PSKFILE\n\n"
"-h          display this help and exit\n"
"-o STRING   set output filename\n";

int psk2code (int argc, char ** argv)
{
  int opt;
  ostream * out = &cout;
  istream * in = &cin;
  while ((opt = getopt (argc, argv, "ho:")) != EOF) {
    switch (opt) {
    case 'h': cout << psk2code_usage; return 0;
    case 'o':
      out = new ofstream (optarg);
      if (out->fail ()) {
	cerr << "psk2code: error opening output file\n";
	return 1;
      }
      break;
    default: break;
    }
  }
  if (optind == argc) {
    cerr << psk2code_usage;
    return 1;
  }
  if (strcmp (argv[optind], "-")) {
    in = new ifstream (argv[optind]);
    if (in->fail ()) {
      cerr << "psk2code: error opening input file\n";
      return 1;
    }
  }
  optind++;
  if (optind < argc)
    cerr << "psk2code: ignoring extra argument(s)\n";
  PSKCode psk (*in);
  Code code (psk);
  (*out) << code;
  return 0;
}

char pskquant_usage[] =
"Usage: pskquant [OPTION...] PSKFILE QUANT\n\n"
"-h          display this help and exit\n"
"-o STRING   set output filename\n";

int pskquant (int argc, char ** argv)
{
  int opt;
  ostream * out = &cout;
  istream * in = &cin;
  while ((opt = getopt (argc, argv, "ho:")) != EOF) {
    switch (opt) {
    case 'h': cout << pskquant_usage; return 0;
    case 'o':
      out = new ofstream (optarg);
      if (out->fail ()) {
	cerr << "pskquant: error opening output file\n";
	return 1;
      }
      break;
    default: break;
    }
  }
  if (optind >= argc-1) {
    cerr << pskquant_usage;
    return 1;
  }
  if (strcmp (argv[optind], "-")) {
    in = new ifstream (argv[optind]);
    if (in->fail ()) {
      cerr << "pskquant: error opening input file\n";
      return 1;
    }
  }
  optind++;
  int quant = atoi (argv[optind++]);
  if (optind < argc)
    cerr << "pskquant: ignoring extra argument(s)\n";
  PSKCode psk (*in);
  psk.Quantize (quant);
  (*out) << psk;
  return 0;
}

char pskinfo_usage[] =
"Usage: pskinfo {PSKFILE | -h}\n"
"-h   display this help and exit\n";

int pskinfo (int argc, char ** argv)
{
  istream * in = &cin;
  if (argc < 2) {
    cerr << pskinfo_usage;
    return 1;
  }
  if (!strcmp (argv[1], "-h")) {
    cout << pskinfo_usage;
    return 0;
  }
  else if (strcmp (argv[1], "-")) {
    in = new ifstream (argv[1]);
    if (in->fail ()) {
      cerr << "pskinfo: error opening input file\n";
      return 1;
    }
  }
  if (argc > 2)
    cerr << "pskinfo: ignoring extra argument(s)\n";
  PSKCode psk (*in);
  cout << "M = " << psk.get_m() << endl
       << "N = 2 * " << psk.get_n() << endl
       << "R = " << (log((double)psk.get_m()) / log(2.0) 
					 / (2*psk.get_n())) << endl
       << "quant = " << psk.get_quant() << endl;
  Code code (psk);
  cout << "Minimal distance for codeword energy 1: " << code.MinDist() << endl;
  return 0;
}

char pskrand_usage[] =
"Usage: pskrand [OPTION...] M N QUANT\n"
"-h          display this help and exit\n"
"-o STRING   set output filename\n"
"-s INT      set random seed\n";

int pskrand (int argc, char ** argv)
{
  int i,j,opt;
  unsigned int seed = (unsigned int) time (NULL);
  ostream * out = &cout;
  while ((opt = getopt (argc, argv, "ho:s:")) != EOF) {
    switch (opt) {
    case 'h': cout << pskrand_usage; return 0;
    case 'o':
      out = new ofstream (optarg);
      if (out->fail ()) {
	cerr << "pskrand: error opening output file\n";
	return 1;
      }
      break;
    case 's': seed = atoi (optarg); break;
    default: break;
    }
  }
  if (optind >= argc-2) {
    cerr << pskrand_usage;
    return 1;
  }
  int m = atoi (argv[optind++]);
  int n = atoi (argv[optind++]);
  int quant = atoi (argv[optind++]);
  if (optind < argc)
    cerr << "pskrand: ignoring extra argument(s)\n";
  PSKCode psk (m, n, quant);
  srand (seed);
  for (i=0; i<m; i++) {
    int * cvi = psk[i];
    for (j=0; j<n; j++)
      cvi[j] = (int) floor (rand () / (RAND_MAX + 1.0) * quant);
  }
  (*out) << psk;
  return 0;
}

char code2psk_usage[] =
"Usage: code2psk [OPTION...] CODEFILE QUANT\n\n"
"-h          display this help and exit\n"
"-o STRING   set output filename\n";

int code2psk (int argc, char ** argv)
{
  int opt;
  ostream * out = &cout;
  istream * in = &cin;
  while ((opt = getopt (argc, argv, "ho:")) != EOF) {
    switch (opt) {
    case 'h': cout << code2psk_usage; return 0;
    case 'o':
      out = new ofstream (optarg);
      if (out->fail ()) {
	cerr << "code2psk: error opening output file\n";
	return 1;
      }
      break;
    default: break;
    }
  }
  if (optind >= argc-1) {
    cerr << code2psk_usage;
    return 1;
  }
  if (strcmp (argv[optind], "-")) {
    in = new ifstream (argv[optind]);
    if (in->fail ()) {
      cerr << "code2psk: error opening input file\n";
      return 1;
    }
  }
  optind++;
  int quant = atoi (argv[optind++]);
  if (optind < argc)
    cerr << "code2psk: ignoring extra argument(s)\n";
  Code code (*in);
  PSKCode psk (code, quant);
  (*out) << psk;
  return 0;
}

char codequant_usage[] =
"Usage: codequant [OPTION...] CODEFILE QUANT\n\n"
"-h          display this help and exit\n"
"-o STRING   set output filename\n";

int codequant (int argc, char ** argv)
{
  int opt;
  ostream * out = &cout;
  istream * in = &cin;
  while ((opt = getopt (argc, argv, "ho:")) != EOF) {
    switch (opt) {
    case 'h': cout << codequant_usage; return 0;
    case 'o':
      out = new ofstream (optarg);
      if (out->fail ()) {
	cerr << "codequant: error opening output file\n";
	return 1;
      }
      break;
    default: break;
    }
  }
  if (optind >= argc-1) {
    cerr << codequant_usage;
    return 1;
  }
  if (strcmp (argv[optind], "-")) {
    in = new ifstream (argv[optind]);
    if (in->fail ()) {
      cerr << "codequant: error opening input file\n";
      return 1;
    }
  }
  optind++;
  int quant = atoi (argv[optind++]);
  if (optind < argc)
    cerr << "codequant: ignoring extra argument(s)\n";
  Code code (*in);
  code.Quantize (quant);
  (*out) << code;
  return 0;
}

char codeinfo_usage[] =
"Usage: codeinfo {CODEFILE | -h}\n"
"-h   display this help and exit\n";

int codeinfo (int argc, char ** argv)
{
  istream * in = &cin;
  if (argc < 2) {
    cerr << codeinfo_usage;
    return 1;
  }
  if (!strcmp (argv[1], "-h")) {
    cout << codeinfo_usage;
    return 0;
  }
  else if (strcmp (argv[1], "-")) {
    in = new ifstream (argv[1]);
    if (in->fail ()) {
      cerr << "codeinfo: error opening input file\n";
      return 1;
    }
  }
  if (argc > 2)
    cerr << "codeinfo: ignoring extra argument(s)\n";
  Code code (*in);
  cout << "M = " << code.get_m() << endl
       << "N = " << code.get_n() << endl
       << "R = " << (log((double)code.get_m()) / log(2.0)
					 / code.get_n()) << endl
       << "Minimal distance: " << code.MinDist () << endl
       << "Maximum codeword energy: " << code.MaxEnergy () << endl
       << "Average codeword energy: " << code.AvgEnergy () << endl;
  code *= sqrt (1.0 / code.AvgEnergy ());
  cout << "Min. distance for avg. energy 1: " << code.MinDist () << endl;
  return 0;
}

char coderand_usage[] =
"Usage: coderand [OPTION...] M N\n"
"-h          display this help and exit\n"
"-o STRING   set output filename\n"
"-s INT      set random seed\n";

int coderand (int argc, char ** argv)
{
  int i,j,opt;
  unsigned int seed = (unsigned int) time (NULL);
  ostream * out = &cout;
  while ((opt = getopt (argc, argv, "ho:s:")) != EOF) {
    switch (opt) {
    case 'h': cout << coderand_usage; return 0;
    case 'o':
      out = new ofstream (optarg);
      if (out->fail ()) {
	cerr << "coderand: error opening output file\n";
	return 1;
      }
      break;
    case 's': seed = atoi (optarg); break;
    default: break;
    }
  }
  if (optind >= argc-1) {
    cerr << coderand_usage;
    return 1;
  }
  int m = atoi (argv[optind++]);
  int n = atoi (argv[optind++]);
  if (optind < argc)
    cerr << "coderand: ignoring extra argument(s)\n";
  Code code (m, n);
  srand (seed);
  for (i=0; i<m; i++) {
    Vector & cvi = code[i];
    for (j=0; j<n; j++)
      cvi[j] = rand () / (RAND_MAX + 1.0);
    cvi /= abs (cvi);
  }
  (*out) << code;
  return 0;
}

int main (int argc, char ** argv)
{
  char * a0 = strrchr (argv[0], '/');
  if (a0 == NULL)
    a0 = argv [0];
  else
    a0++;
  if (!strcmp (a0, "psk2code"))
    return psk2code (argc, argv);
  else if (!strcmp (a0, "pskquant"))
    return pskquant (argc, argv);
  else if (!strcmp (a0, "pskinfo"))
    return pskinfo (argc, argv);
  else if (!strcmp (a0, "pskrand"))
    return pskrand (argc, argv);
  else if (!strcmp (a0, "code2psk"))
    return code2psk (argc, argv);
  else if (!strcmp (a0, "codequant"))
    return codequant (argc, argv);
  else if (!strcmp (a0, "codeinfo"))
    return codeinfo (argc, argv);
  else if (!strcmp (a0, "coderand"))
    return coderand (argc, argv);
  else {
    cerr << a0 << " cannot be run directly\n";
    return 1;
  }
}
