// Test program for treedecoder.h
// Usage: fanotest [dmin [n1 [n [erasures]]]

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// $Id: fanotest.cc,v 1.1 2003/10/14 08:59:11 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <math.h>
#include <iostream>
#include <stdlib.h>
#include <time.h>

#include "xrscode.h"
#include "treedecoder.h"
#include "gf.h"

using namespace std;

#ifdef GF16
#include "gf16tables.h"
typedef GF<GF16Tables> GFq;
#else
#ifdef GF32
#include "gf32tables.h"
typedef GF<GF32Tables> GFq;
#else
#ifdef GF64
#include "gf64tables.h"
typedef GF<GF64Tables> GFq;
#else
#ifdef GF128
#include "gf128tables.h"
typedef GF<GF128Tables> GFq;
#else
#include "gf256tables.h"
typedef GF<GF256Tables> GFq;
#endif
#endif
#endif
#endif

typedef XRSCode<GFq> XRSC;
typedef TreeDecoder<GFq,XRSC> TREEDEC;
typedef double * pDouble;

const int loops = 100; // number of iterations per output line

#ifndef NDEBUG
void PrintVec (char * title, GFq * vec, int len)
{
  int i;
  cerr << title << " = [";
  if (len > 0)
    cerr << (int) vec[0];
  for (i=1; i<len; i++)
    cerr << ',' << (int) vec[i];
  cerr << "]\n";
}
#endif

int main (int argc, char ** argv)
{
  int q, d, xd, n, k, eras, err, i, j, l;
  q = GFq::q();
  d = 5;
  xd = 4;
  n = q - 1;
  eras = 0;
  if (argc > 1) {
    d = atoi (argv [1]);
    if (argc > 2) {
      xd = atoi (argv[2]);
      if (argc > 3) {
	n = atoi (argv [3]);
	if (argc > 4)
	  eras = atoi (argv [4]);
      }
    }
  }
  XRSC rsc (d, xd, n);
  TREEDEC tdec (rsc);
  tdec.AllocateStack (1000);
  k = rsc.get_k();
  cout << "q = " << q << ", n = " << n << ", xd = " << xd
       << ", k = " << k << ", dmin = " << d
       << ("\n\ninput\t\toutput\n"
	   "eras's\terrors\teras's\terrors\ttotal\n\n");
  GFq * info = new GFq [k];
  GFq * sent = new GFq [n];
  GFq * rcvd = new GFq [n];
  pDouble * alpha = new pDouble [n];
  for (i=0; i<n; i++)
    alpha[i] = new double [q];
  // setup erasure/error location vector
  int * I = new int [n];
  for (i=0; i<n; i++)
    I[i] = i;
  srand (time (NULL));
  for (err = 0; eras+err <= n; err++) {
    int eras1 = 0, err1 = 0;
    for (i = 0; i < loops; i++) {
      // generate random information word
      for (j = 0; j < k; j++)
	info[j] = (unsigned char) floor ((double)q*rand()/(RAND_MAX+1.0));
#ifndef NDEBUG
      PrintVec ("info", info, k);
#endif
      rsc.Encode (sent, info);
#ifndef NDEBUG
      PrintVec ("sent", sent, n);
#endif
      // scramble erasure/error location vector
      for (j = 0; j < n; j++) {
	int j1 = (int) floor ((double)n*rand()/(RAND_MAX+1.0));
	int h = I[j];
	I[j] = I[j1];
	I[j1] = h;
      }
      // copy sent[] to rcvd[]
      for (j = 0; j < n; j++)
	rcvd[j] = sent[j];
      // modify rcvd[I[0]]..rcvd[I[eras+err-1]]
      for (j = 0; j < eras+err; j++)
	rcvd[I[j]] +=
	  GFq (1 + (int) floor ((double)(q-1)*rand()/(RAND_MAX+1.0)));
#ifndef NDEBUG
      PrintVec ("rcvd", rcvd, n);
#endif
      // setup alpha[]
      for (j = 0; j < n; j++) {
	for (l = 0; l < q; l++)
	  alpha[j][l] = -999;
	alpha[j][rcvd[j]] = 0;
      }
      int res = tdec.StackCorrect (rcvd, rcvd, (const double **)alpha);
//       int res = tdec.FanoCorrect (rcvd, rcvd, alpha, 4.0, 100);
#ifndef NDEBUG
      cerr << "res = " << res << ", ";
      PrintVec ("rcvd", rcvd, n);
#endif
      if (res < 0)
	eras1++;
      else {
	int e = 0;
	for (j=0; j<n; j++)
	  if (sent[j] != rcvd[j])
	    e++;
#ifndef NDEBUG
	cerr << e << " errors\n";
#endif
	err1 += e;
      }
    }
    cout << eras << '\t' << err << '\t'
	 << (double)eras1 / (double)loops << '\t'
	 << (double)err1 / (double)loops << '\t'
	 << (double)((eras+err)*eras1 + err1) / (double)loops << endl;
  }
  delete [] I;
  for (i=0; i<n; i++)
    delete [] alpha[i];
  delete [] alpha;
  tdec.DeleteStack ();
  delete [] rcvd;
  delete [] sent;
  delete [] info;
}
