#ifndef _MLCG_H
#define _MLCG_H

// A Multiplicative Linear Congruential Generator
// for generating uniformly distributed pseudorandom numbers

// The following is mainly taken from the GNU C++ library,
// but I've modified it to use templates rather than overloaded virtual
// functions for speed reasons

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// Created: 12/15/1996
// $Id: mlcg.h,v 1.1 2003/10/14 08:59:11 frank Exp $

// $Log: mlcg.h,v $
// Revision 1.1  2003/10/14 08:59:11  frank
// moved from . to src/
//
// Revision 1.3  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.2  1998/05/28 12:57:01  paehlke
// Final version for masters thesis
//
// Revision 1.1  1997/06/20 18:06:50  frank
// Initial revision
//

// --- BEGIN OF PUBLIC INTERFACE ---

#ifndef _MATH_H
#include <math.h>
#endif

#ifndef _G_int32_t
typedef int _G_int32_t;	// this is only legal for 32 bit machines!
typedef unsigned int _G_uint32_t;
#endif

class MLCG {
public:

  MLCG(_G_int32_t seed1 = 0, _G_int32_t seed2 = 1);

  // Return a long-words word of random bits:

  _G_uint32_t asLong();

  // Return uniformly distributed pseudorandom numbers
  // from the interval [0,1]:

  double asDouble();

  void reset();
  _G_int32_t seed1();
  void seed1(_G_int32_t);	// modify seed1 + return old value
  _G_int32_t seed2();
  void seed2(_G_int32_t);	// modify seed2 + return old value
  void reseed(_G_int32_t, _G_int32_t);	// modify seed1 + seed2

  // --- END OF PUBLIC INTERFACE ---

private:

  _G_int32_t initialSeedOne;
  _G_int32_t initialSeedTwo;
  _G_int32_t seedOne;
  _G_int32_t seedTwo;
};

inline MLCG::MLCG(_G_int32_t seed1, _G_int32_t seed2)
{
  initialSeedOne = seed1;
  initialSeedTwo = seed2;
  reset();
}

inline _G_int32_t MLCG::seed1()
{
  return(seedOne);
}

inline void MLCG::seed1(_G_int32_t s)
{
  initialSeedOne = s;
  reset();
}

inline _G_int32_t MLCG::seed2()
{
  return(seedTwo);
}

inline void MLCG::seed2(_G_int32_t s)
{
  initialSeedTwo = s;
  reset();
}

inline void MLCG::reseed(_G_int32_t s1, _G_int32_t s2)
{
  initialSeedOne = s1;
  initialSeedTwo = s2;
  reset();
}

inline double MLCG::asDouble()
{
  return ldexp (asLong(), -31);
}

#endif
