#ifndef _NORMAL_H
#define _NORMAL_H

// Templates for generating normal distributed pseudorandom numbers

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// Created: 01/12/1997
// $Id: normal2.h,v 1.1 2003/10/14 08:59:11 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: normal2.h,v $
// Revision 1.1  2003/10/14 08:59:11  frank
// moved from . to src/
//
// Revision 1.3  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.2  1998/05/28 12:57:03  paehlke
// Final version for masters thesis
//
// Revision 1.1  1997/06/20 18:15:44  frank
// Initial revision
//

// --- BEGIN OF PUBLIC INTERFACE ---

template<class RNG> class Normal
{
public:

  Normal (RNG *gen, double xmean=0.0, double xvariance=1.0);
  double mean();
  double mean(double x);	// modify mean + return previous value
  double variance();
  double variance(double x);	// modify variance + return previous value
  virtual double operator()();	// the actual generator function

// --- END OF PUBLIC INTERFACE ---

private:

  RNG * pGenerator;
  double pMean;
  double pVariance;
  double pStdDev;
  char haveCachedNormal;
  double cachedNormal;
};

template<class RNG> inline Normal<RNG>::Normal
(RNG * gen, double xmean, double xvariance)
{
  pGenerator = gen;
  pMean = xmean;
  pVariance = xvariance;
  pStdDev = sqrt(pVariance);
  haveCachedNormal = 0;
}

template<class RNG> inline double Normal<RNG>::mean() { return pMean; };

template<class RNG> inline double Normal<RNG>::mean(double x)
{
  double t=pMean; pMean = x;
  return t;
}

template<class RNG> inline double Normal<RNG>::variance() { return pVariance; }

template<class RNG> inline double Normal<RNG>::variance(double x)
{
  double t=pVariance; pVariance = x;
  pStdDev = sqrt(pVariance);
  return t;
};

// The rational approximation for the normal CDF comes from
// Abramovitz/Stegun, 'Handbook of mathematical formulae', formula 26.2.23

template<class RNG> double Normal<RNG>::operator () ()
{
  double p = pGenerator -> asDouble();
  if (p<=.5) {
    double t = sqrt(-2*log(p));
    return (t-((.010328*t+.802853)*t+2.515517)/
	    (((.001308*t+.189269)*t+1.432788)*t+1));
  }
  else {
    p = 1.0-p;
    double t = sqrt(-2*log(p));
    return (((.010328*t+.802853)*t+2.515517)/
	    (((.001308*t+.189269)*t+1.432788)*t+1)-t);
  }
}

#endif
