#ifndef _OPTIMIZE_H
#define _OPTIMIZE_H

// Function templates for threshold acceptance
// and steepest ascent optimization

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// Created: 01/19/1997
// $Id: optimize.h,v 1.1 2003/10/14 08:59:11 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: optimize.h,v $
// Revision 1.1  2003/10/14 08:59:11  frank
// moved from . to src/
//
// Revision 1.4  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.3  1998/05/28 12:57:03  paehlke
// Final version for masters thesis
//
// Revision 1.2  1997/06/20 14:19:47  frank
// added CCLASS to template parameters
//
// Revision 1.1  1997/06/20 12:54:16  frank
// Initial revision
//

#ifndef _CODE_H
#include "code.h"
#endif

// Optimize_TA tries to maximize MinSqrdist () using a
// threshold acceptance algorithm.

template<class OCLASS, class CCLASS>
void Optimize_TA (OCLASS & code, CCLASS & opt_code,
		  double thresh_start, double thresh_end, double thresh_mult,
		  double alter_dist)
{
  int loops, i;
  double ov = code.OptValue ();
  double opt_ov = ov;
  double thresh = thresh_start; // max. acceptable threshold
  cerr << "# of iterations\tthresh\t\tmd\t\tmd_opt\n\n";
  loops = 0;
  while (thresh >= thresh_end) {
    for (i=0; i<10000; i++) {
      double old_ov = ov;
      code.DoChange (alter_dist);
      ov = code.OptValue ();
      if (ov < old_ov - thresh) {
        code.UndoChange ();
        ov = old_ov;
      }
      else if (ov > opt_ov) {
        opt_code = code;
	opt_ov = ov;
      }
    }
    loops ++;
    cerr << loops << "0000\t\t" << thresh << "\t\t"
	 << ov << "\t\t" << opt_ov <<  endl;
    thresh *= thresh_mult;
  }
  cerr << endl;
}

// Phase 2 of the optimization process takes p_opt[]
// and performs a simple steepest ascent optimization

template<class OCLASS, class CCLASS>
void Optimize_Phase2 (OCLASS & code, CCLASS & opt_code)
{
  int loops, i;
  double epsilon = .1, opt_ov;
  cerr << "Phase 2 ... \n";
  opt_ov = code.OptValue ();
  loops=0;
  cerr << "# of iterations\tepsilon\t\tov\t\topt_ov\n";
  for (loops=1; loops<=100 && epsilon > 1e-7; loops++) {
    for (i=0; i<100; i++) {
      double old_ov = code.OptValue();
      code.SteepestAscent (epsilon);
      if (code.OptValue() <= old_ov)
	epsilon *= .995;
      else if (code.OptValue() > opt_ov) {
        opt_code = code;
        opt_ov = code.OptValue();
      }
    }
    cerr << loops << "00\t\t" << epsilon << '\t'
	 << code.OptValue() << '\t' << opt_ov << endl;
  }
  cerr << "Finished\n";
}

#endif
