// Programm zur Berechnung von Koordinatenlisten fuer diverse
// Funktionsgraphen als Eingabe fuer gnuplot.
// (Quick and dirty - um eine andere Funktion zu plotten, ist ein
//  erneutes Kompilieren notwendig)

// Autor: Frank Paehlke <Frank.Paehlke@stud.uni-karlsruhe.de>
// $Id: plot.cc,v 1.1 2003/10/14 08:59:11 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: plot.cc,v $
// Revision 1.1  2003/10/14 08:59:11  frank
// moved from . to src/
//
// Revision 1.3  2003/10/14 08:41:58  frank
// ANSI C++ / gcc 3.x compatibility changes
//
// Revision 1.2  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.1  1998/05/28 12:56:37  paehlke
// Final version for masters thesis
//

#include <math.h>
#include <iostream>
#include <stdlib.h>
#include <assert.h>
#include "kanalkap.h"

using namespace std;

// Hoehe eines Dreiecks mit Grundlinie ((x0,y0), (x1,y1))
// und gegenueberliegendem Eckpunkt (x,y)

double deviation (double x0, double y0, double x1, double y1,
		  double x, double y)
{
  double v1x = x1 - x0, v1y = y1 - y0;
  double v2x = x - x0, v2y = y - y0;
  double d = (v1x*v2x + v1y*v2y) / (v1x*v1x + v1y*v1y);
  v2x -= d * v1x; v2y -= d * v1y;
  return hypot (v2x, v2y);
}

void plot2 (double (* fx) (double), double (* fy) (double),
	    double t0, double fxt0, double fyt0,
	    double t1, double fxt1, double fyt1,
	    double epsilon,
	    void (*out) (double, double))
{
  double t = (t0 + t1)/2;
  double fxt = (*fx) (t), fyt = (*fy) (t);
  if (deviation (fxt0, fyt0, fxt1, fyt1, fxt, fyt) >= epsilon) {
    plot2 (fx, fy, t0, fxt0, fyt0, t, fxt, fyt, epsilon, out);
    plot2 (fx, fy, t, fxt, fyt, t1, fxt1, fyt1, epsilon, out);
  }
  else {
    (*out) (fxt0, fyt0);
    (*out) (fxt, fyt);
  }
}

void plot (double (* fx) (double), double (* fy) (double),
	   double _t0, double _t1,
	   int n0, double epsilon,
	   void (*out) (double, double))
{
  assert ((n0 > 0) && (epsilon > 0));
  int i;
  double step = (_t1 - _t0) / n0;
  double t0, t1, fxt0, fyt0, fxt1, fyt1;
  t0 = _t0; fxt0 = (*fx) (t0); fyt0 = (*fy) (t0);
  fxt1 = fyt1 = 0.0;
  for (i=1; i <= n0; i++) {
    t1 = _t0 + step * i;
    fxt1 = (*fx) (t1); fyt1 = (*fy) (t1);
    plot2 (fx, fy, t0, fxt0, fyt0, t1, fxt1, fyt1, epsilon, out);
    t0 = t1; fxt0 = fxt1; fyt0 = fyt1;
  }
  (*out) (fxt1, fyt1);
}

void printout (double x, double y)
{
  cout << x << '\t' << y << endl;
}

double fx (double t)
{
  //  return t / (2 * awgn_r0_snr  (t));
  // return to_db (t / (2 * awgn_r0_snr (t)));
  //  if (t==0) return 2*M_LN2; else return to_db(t/(2*awgn_r0_snr(t)));
  return t;
}

double fy (double t)
{
  return bsc_eelow_p_r(.01,t);
  //  if (t==0.0) return 1.0;
  //  double a=0.5*(sqrt((8+t)*t)-t);
  //double r=t/a;
  //return (1-a)*((1-r)*(1-r));
}

double ifunc (double x)
{
  return sin(x);
}

const double x0 = bsc_rcrit_p(.01);
const double x1 = bsc_rc_p(.01);

double R;
double er;

double tosolve1 (int q, double p)
{
  return usc_eehigh_p_r(q,p,R);
}

double tosolve2 (int q, double p)
{
  return usc_eelow_p_r(q,p,R);
}

int main (int argc, char ** argv)
{
  double epsilon = 0.1;
  if (argc > 1)
    epsilon = atof (argv [1]);
  cout << "#R_ex = " << bsc_rex_p(.01) << endl;
  cout << "#R_crit = " << bsc_rcrit_p(.01) << endl;
  cout << "#R_0 = " << bsc_r0_p(.01) << endl;
  cout << "#R_c = " << bsc_rc_p(.01) << endl;
  cout << "#E(0) = " << bsc_eelow_p_r(.01,0) << endl;
  plot (fx, fy, x0, x1, 2, epsilon, printout);
  exit (0);
  int i,j;
  for (j=10; j<100; j++) {
    double r = j/10000.0, s = awgnbin_snr_rc (r);
    cout << j/10000.0 << '\t' << s << '\t'
	 << to_db(s) << '\t' << s/(2*r) << '\t'
	 << to_db(s/(2*r)) << endl;
  }
  for (j=10; j<100; j++) {
    double r = j/1000.0, s = awgnbin_snr_rc (r);
    cout << j/1000.0 << '\t' << s << '\t'
	 << to_db(s) << '\t' << s/(2*r) << '\t'
	 << to_db(s/(2*r)) << endl;
  }
  for (j=10; j<90; j++) {
    double r = j/100.0, s = awgnbin_snr_rc (r);
    cout << j/100.0 << '\t' << s << '\t'
	 << to_db(s) << '\t' << s/(2*r) << '\t'
	 << to_db(s/(2*r)) << endl;
  }
  for (j=900; j<990; j++) {
    double r = j/1000.0, s = awgnbin_snr_rc (r);
    cout << j/1000.0 << '\t' << s << '\t'
	 << to_db(s) << '\t' << s/(2*r) << '\t'
	 << to_db(s/(2*r)) << endl;
  }
  for (j=9900; j<10000; j++) {
    double r = j/10000.0, s = awgnbin_snr_rc (r);
    cout << j/10000.0 << '\t' << s << '\t'
	 << to_db(s) << '\t' << s/(2*r) << '\t'
	 << to_db(s/(2*r)) << endl;
  }
  return 0;
}
