// Test program for rscode.h
// Usage: rstest [dmin [n [erasures]]

// (C) 1997-1998 Frank Paehlke <frank@paehlke.de>
// $Id: rstest.cc,v 1.1 2003/10/14 08:59:11 frank Exp $

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

// $Log: rstest.cc,v $
// Revision 1.1  2003/10/14 08:59:11  frank
// moved from . to src/
//
// Revision 1.5  2003/10/14 08:41:58  frank
// ANSI C++ / gcc 3.x compatibility changes
//
// Revision 1.4  2002/08/02 14:45:06  paehlke
// added GPL comment
//
// Revision 1.3  1998/05/28 12:56:38  paehlke
// Final version for masters thesis
//
// Revision 1.2  1997/10/19 16:28:58  frank
// added comments, separated debug output from regular output
//
// Revision 1.1  1997/06/20 18:50:12  frank
// Initial revision
//

#include <math.h>
#include <iostream>
#include <stdlib.h>
#include <time.h>

#include "rscode.h"
#include "gf.h"

using namespace std;

#ifdef GF16
#include "gf16tables.h"
typedef GF<GF16Tables> GFq;
#else
#ifdef GF32
#include "gf32tables.h"
typedef GF<GF32Tables> GFq;
#else
#ifdef GF64
#include "gf64tables.h"
typedef GF<GF64Tables> GFq;
#else
#ifdef GF128
#include "gf128tables.h"
typedef GF<GF128Tables> GFq;
#else
#include "gf256tables.h"
typedef GF<GF256Tables> GFq;
#endif
#endif
#endif
#endif

typedef RSCode<GFq> RSC;

const int loops = 100; // number of iterations per output line

#ifndef NDEBUG
void PrintVec (char * title, GFq * vec, int len)
{
  int i;
  cerr << title << " = [";
  if (len > 0)
    cerr << (int) vec[0];
  for (i=1; i<len; i++)
    cerr << ',' << (int) vec[i];
  cerr << "]\n";
}
#endif

main (int argc, char ** argv)
{
  int q, d, n, k, eras, err, i, j;
  q = GFq::q();
  d = 5;
  n = q - 1;
  eras = 0;
  if (argc > 1) {
    d = atoi (argv [1]);
    if (argc > 2) {
      n = atoi (argv [2]);
      if (argc > 3)
	eras = atoi (argv [3]);
    }
  }
  RSC rsc (d, n);
  k = rsc.get_k();
  cout << "q = " << q << ", n = " << n
       << ", k = " << k << ", dmin = " << d
       << ("\n\ninput\t\toutput\n"
	   "eras's\terrors\teras's\terrors\ttotal\n\n");
  GFq * info = new GFq [k];
  GFq * sent = new GFq [n];
  GFq * rcvd = new GFq [n];
  // setup erasure/error location vector
  int * I = new int [n];
  for (i=0; i<n; i++)
    I[i] = i;
  srand (time (NULL));
  for (err = 0; eras+err <= n; err++) {
    int eras1 = 0, err1 = 0;
    for (i = 0; i < loops; i++) {
      // generate random information word
      for (j = 0; j < k; j++)
	info[j] = (unsigned char) floor ((double)q*rand()/(RAND_MAX+1.0));
#ifndef NDEBUG
      PrintVec ("info", info, k);
#endif
      rsc.Encode (sent, info);
#ifndef NDEBUG
      PrintVec ("sent", sent, n);
#endif
      // scramble erasure/error location vector
      for (j = 0; j < n; j++) {
	int j1 = (int) floor ((double)n*rand()/(RAND_MAX+1.0));
	int h = I[j];
	I[j] = I[j1];
	I[j1] = h;
      }
      // copy sent[] to rcvd[]
      for (j = 0; j < n; j++)
	rcvd[j] = sent[j];
      // modify rcvd[I[0]]..rcvd[I[eras+err-1]]
      for (j = 0; j < eras+err; j++)
	rcvd[I[j]] +=
	  GFq (1 + (int) floor ((double)(q-1)*rand()/(RAND_MAX+1.0)));
#ifndef NDEBUG
      PrintVec ("rcvd", rcvd, n);
#endif
      int res = rsc.Correct (rcvd, rcvd, I, eras);
#ifndef NDEBUG
      cerr << "res = " << res << ", ";
      PrintVec ("rcvd", rcvd, n);
#endif
      if (res < 0)
	eras1++;
      else {
	int e = 0;
	for (j=0; j<n; j++)
	  if (sent[j] != rcvd[j])
	    e++;
#ifndef NDEBUG
	cerr << e << " errors\n";
#endif
	err1 += e;
      }
    }
    cout << eras << '\t' << err << '\t'
	 << (double)eras1 / (double)loops << '\t'
	 << (double)err1 / (double)loops << '\t'
	 << (double)((eras+err)*eras1 + err1) / (double)loops << endl;
  }
  delete [] I;
  delete [] rcvd;
  delete [] sent;
  delete [] info;
}
